#!/usr/bin/env python
#
# A reasonably generic probe framework
#
# James Casey <james.casey@cern.ch>
#

import os
import sys
import lfc
import getopt
import pydoc
import time
import socket
import errno

def time_now():
    try:
        from datetime import datetime
        isotime = datetime.utcnow()
        return isotime.isoformat()
    except ImportError:
        try:
            from xml.utils.iso8601 import tostring as toiso8601
            return toiso8601(time.time())
        except ImportError:
            return time.strftime("%Y-%m-%dT%H:%M:%SZ")

def to_status(code):
    if code in (0, 'OK', 'ok', 'Ok'):
        return 'OK'
    elif code in (1, 'WARNING', 'warning', 'Warning'):
        return 'WARNING'
    elif code in (2, 'CRITICAL', 'critical', 'Critical'):
        return 'CRITICAL'
    else:
        return 'UNKNOWN'
    
     
class ProbeFormatRenderer :
    """ A class to abstract out the rendering of the test results"""

    def __init__(self, stream = sys.stdout):
        self.output = stream

    def render(self, tuples = {}):
        saveout = sys.stdout
        sys.stdout = self.output

        for attr,value in tuples.items():
            if attr != 'detailsData':
                print "%s: %s"%(attr, value)
        if tuples.has_key('detailsData'):
            print "%s: %s"%('detailsData', tuples['detailsData'])
        print "EOT"
        sys.stdout = saveout


class MetricGatherer :
    """A Base class for Metrics.  All methods starting with 'metric'
    are considered metrics with the signature:
    (RETVAL, DESC) = metricFoo()
    """

    def __init__(self, type):
        self.description = {}
        self.methodMap = {}
        self.serviceType = type

    def list(self):
        "List all metric methods in this class"
        metrics = []
        for k,v in self.description.items():
            metrics.append(k)
        return metrics

    def gather(self, metric):
        "Run a given metric instance, and return the result"
        if self.methodMap.has_key(metric):
            methodName = self.methodMap[metric]
        else:
            methodName = "metric" + metric
        timestamp = time_now()
        if hasattr(self,methodName):
            try:
                ret = getattr(self,methodName)()
                return {'metricStatus' : to_status(ret[0]), 'summaryData' : ret[1],
                        'timestamp' : timestamp, 'serviceType' : self.serviceType,
                        'metricName' : metric}
            except TypeError, e:
                # arguments mismatch
                return {'timestamp' : timestamp, 'serviceType' : self.serviceType,
                        'metricName' : metric, 'metricStatus' : to_status(3),
                        'summaryData' : "Internal argument mismatch : %s"%e}
        else:
            return {'timestamp' : timestamp, 'serviceType' : self.serviceType,
                    'metricName' : metric, 'metricStatus' : to_status(3), 
                    'summaryData' : "Metric does not exist"}

    def desc(self, metric):
        "Return the test definition block"
        desc = None
        if self.description.has_key(metric):
            desc = {}
            for k,v in self.description[metric].items():
                desc[k] = v
            desc['serviceType'] = self.serviceType
        return desc
            

    def __repr__(self):
        return "<%s metrics=%s>"% (self.__class__, self.list())

class Runner:
    def __init__(self, gathererClass, renderer = ProbeFormatRenderer()):
        self.gathererClass = gathererClass
        self.renderer = renderer

    def run(self, argv):
        usage="""Usage:
%s [-v|--vo <voName>]  -u|--uri <URI> -m|--metric <name>  [-t||--timeout secs]
        
You must specify a metric name (-m) and URI (-u)
        
"""%argv[0]
            
        specversion = '0.91'
       
        opts,args= (None,None)
        try:
            opts,args=getopt.getopt(argv[1:],'v:s:u::m:t:lh',
                                    ['vo=','service=','uri=', 'metric=','timeout=','list','help'])
        except getopt.GetoptError, e:
            sys.stdout.write(usage)
            sys.stdout.write("Error : %s\n"% e)
            sys.exit(0)
        
        opts_map={ '-v': 'voName', '--vo': 'voName',
                   '-s': 'serviceName', '--service': 'serviceName'}
	   

        tuples={}
        metric=None
        list_metrics=None
        all=None
        help=None
        timeout=None
        
        tuples['voName']='ops'
        
        for o,v in opts:
            if o in ('--help', '-h'):
                help=1
            if o in ('-l','--list'):
                list_metrics=1
            if o in ('-m','--metric'):
                metric=v
            if o in ('-u', '--uri'):
                tuples['serviceURI']=v
            if o in ('-t', '--timeout'):
                timeout=v
            if o in ('-n','--node'):
                tuples[opts_map[o]]=v
            if o in ('-v','--vo'):
                tuples[opts_map[o]]=v

        if tuples.has_key('serviceURI'):
            tuples['gatheredAt'] = socket.gethostname()
        else:
            # UGH ! Need to do this, just to pull some version stuff
            # out of the probe
            tuples['serviceURI']= 'http://foo.example.com:8888/'
            gatherer = self.gathererClass(tuples, timeout)
            if help:
                sys.stderr.write("serviceType: %s\n"%gatherer.serviceType)
                sys.stderr.write("probeSpecificationVersion: %s\n"%specversion)
                for k,v in gatherer.probeinfo.items():
                    sys.stderr.write("%s: %s\n"%(k,v))
                sys.stderr.write('\n')
            	sys.stderr.write(usage)
            	sys.exit(1)
            elif list_metrics:
                for metric in gatherer.list():
                    self.renderer.render(gatherer.desc(metric))
                sys.exit(1)

        gatherer = self.gathererClass(tuples, timeout)


        # and now gather the metric, if specifed
        if not metric:
            sys.stderr.write(usage)
            sys.stderr.write('Available metrics are:\n')
            for metric in gatherer.list():
                sys.stderr.write('\t%s\n'%metric)
            print >>sys.stderr
            sys.exit(1)

        result = gatherer.gather(metric)
        # Add command line info, which should go into rendered output
        result.update(tuples)
        self.renderer.render(result)
        if to_status(result['metricStatus']) == 'Unknown':
            return 1

        return 0
